
#ifndef QUEUE_H
#define QUEUE_H

#include <stddef.h>

#define MUTEX_CREATE(x) {x=CreateMutex(NULL,FALSE,NULL);}
#define MUTEX_LOCK(x)   {if(x)WaitForSingleObject(x,INFINITE);}
#define MUTEX_UNLOCK(x) {if(x)ReleaseMutex(x);}
#define MUTEX_DELETE(x) {if(x){CloseHandle(x);x=NULL;}}


typedef struct _LIST_HEAD
{
    struct _LIST_HEAD*	pNext;
    struct _LIST_HEAD*  pPrev;
}LIST_HEAD, *PLIST_HEAD, **PPLIST_HEAD;
 

#define INIT_LIST_HEAD(ptr) \
do { \
    (ptr)->pNext = (ptr); (ptr)->pPrev = (ptr); \
} while (0)


#define LIST_ADD(new, prev, next) \
do { \
    PLIST_HEAD    pPrev = prev; \
    PLIST_HEAD    pNext = next; \
    pNext->pPrev = new; \
	(new)->pNext = pNext; \
	(new)->pPrev = pPrev; \
	pPrev->pNext = new; \
} while (0)

#define LIST_ADD_HEAD(new, head)        LIST_ADD(new, head, (head)->pNext)

#define LIST_ADD_TAIL(new, head)        LIST_ADD(new, (head)->pPrev, head)

#define LIST_UNLINK(prev, next) \
do { \
    (next)->pPrev = prev; \
    (prev)->pNext = next; \
} while (0)

#define LIST_DEL(entry) \
do { \
	if ( (entry)->pPrev && (entry)->pNext )\
	{\
		LIST_UNLINK((entry)->pPrev, (entry)->pNext); \
		(entry)->pPrev = NULL; \
		(entry)->pNext = NULL; \
	}\
} while (0)

#define LIST_EMPTY(head)            ((head)->pNext == head)

//---------------------------------------- Version 1.1 -------------------------------------------------------
//#define LIST_ENTRY_T(ptr,type,member)      ((type *)((char *)(ptr)-(unsigned long)(&((type *)0)->member)))
#define LIST_ENTRY_T(ptr,type,member)        (type *)((char *)ptr - offsetof(type,member))
//---------------------------------------- Version 1.1 -------------------------------------------------------

#define LIST_FIRST(head)                (((head)->pNext != (head)) ? (head)->pNext: NULL)




#endif // QUEUE_H
